@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.33 2004/01/05 19:15:23 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.38 2004/02/17 17:25:44 richb Exp $
  *
  *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -390,20 +390,7 @@ do_delete()     /* Remove the last numeric character typed. */
 
     len = strlen(v->display);
     if (len > 0) {
-        size_t l2;
-
-	l2 = strlen(v->radix);
-        if (len >= l2 && memcmp(v->display + len - l2, v->radix, l2) == 0) {
-	    v->display[len-l2] = '\0';
-	} else {
-	    l2 = strlen(v->tsep);
-	    if (len >= l2
-		&& memcmp(v->display + len - l2, v->tsep, l2) == 0) {
-	        v->display[len-l2] = '\0';
-	    } else {
-	        v->display[len-1] = '\0';
-	    }
-	}
+	v->display[len-1] = '\0';
     }
 
 /*  If we were entering a scientific number, and we have backspaced over
@@ -417,7 +404,7 @@ do_delete()     /* Remove the last numeric character typed. */
 
 /* If we've backspaced over the numeric point, clear the pointed flag. */
 
-    if (v->pointed && !(strstr(v->display, v->radix))) {
+    if (v->pointed && !(strchr(v->display, '.'))) {
         v->pointed = 0;
     }
 
@@ -426,7 +413,6 @@ do_delete()     /* Remove the last numeric character typed. */
 
     if (v->dtype == FIX) {
         STRCPY(v->fnum, v->display);
-        add_tsep();
         set_display(v->fnum);
     }
 }
@@ -447,15 +433,12 @@ do_exchange()         /* Exchange display with memory register. */
 void
 do_expno()           /* Get exponential number. */
 {
-    v->pointed = (strstr(v->display, v->radix) != NULL);
+    v->pointed = (strchr(v->display, '.') != NULL);
     if (!v->new_input) {
         STRCPY(v->display, "1.0 +");
         v->new_input = v->pointed = 1;
     } else if (!v->pointed) {
-        char str[4];
-
-        SPRINTF(str, "%s +", v->radix);
-        STRNCAT(v->display, str, 3);
+        STRNCAT(v->display, ". +", 3);
         v->pointed = 1;
     } else if (!v->key_exp) {
         STRNCAT(v->display, " +", 2);
@@ -504,7 +487,8 @@ do_factorial(int *MPval, int *MPres)
                 mpmuli(MPa, &i, MPa);
                 mpcmd(MPa, &val);
                 if (v->error) {
-                    break;
+                    mperr();
+                    return;
                 }
                 i--;
             }
@@ -623,7 +607,7 @@ do_mode()                   /* Set special calculator mode. */
 {
     char title[MAXLINE];
 
-    SPRINTF(title, "%s [%s]", v->tool_label, _(mstrs[(int) v->modetype]));
+    SPRINTF(title, "%s- %s", v->tool_label, _(mstrs[(int) v->modetype]));
     set_title(FCP_KEY, title);
     put_resource(R_MODE, Rmstr[(int) v->modetype]);
     set_mode(v->modetype);
@@ -798,10 +782,10 @@ do_point()                   /* Handle numeric point. */
 {
     if (!v->pointed) {
         if (v->toclear) {
-            STRCPY(v->display, v->radix);
+            STRCPY(v->display, ".");
             v->toclear = 0;
         } else {
-            STRCAT(v->display, v->radix);
+            STRCAT(v->display, ".");
         }
         v->pointed = 1;
     }
