@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.104 2003/11/07 18:47:30 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.114 2004/02/09 15:56:45 richb Exp $
  *
  *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+#include <limits.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <netdb.h>
@@ -86,9 +87,9 @@ typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *fin_buttons[FROWS * FCOLS];
     GtkWidget *sci_buttons[SROWS * SCOLS];
 
-    GtkWidget *bas_frame;
-    GtkWidget *fin_frame;
-    GtkWidget *sci_frame;
+    GtkWidget *bas_panel;
+    GtkWidget *fin_panel;
+    GtkWidget *sci_panel;
 
     Display *dpy;
 
@@ -117,7 +118,7 @@ typedef struct {
 static GtkWidget *create_menu(enum menu_type, struct button *);
 static GtkWidget *create_mode_panel(GtkWidget *);
 static GtkWidget *make_menu_button(gchar *, int);
-static GtkWidget *make_but_frame(GtkWidget *, GtkWidget **,
+static GtkWidget *make_but_panel(GtkWidget *, GtkWidget **,
                                  struct button *, int, int, char *);
 
 static char *make_hostname(Display *);
@@ -153,8 +154,10 @@ static void reset_mode_values(enum mode_type);
 static void set_button_state(GtkWidget *, int);
 static void set_gcalctool_icon(void);
 static void set_memory_toggle(int);
+static void set_show_tsep_toggle(int);
 static void set_show_zeroes_toggle(int);
 static void trig_cb(GtkToggleButton *, gpointer);
+static void ts_proc(gpointer, int, GtkWidget *);
 
 static XVars X;
 
@@ -168,7 +171,7 @@ static GtkItemFactoryEntry main_menu[] = {
     { N_("/Edit/_Copy"),           NULL, mb_proc, M_COPY,  "<StockItem>", GTK_STOCK_COPY },
     { N_("/Edit/_Paste"),          NULL, mb_proc, M_PASTE, "<StockItem>", GTK_STOCK_PASTE },
     { N_("/Edit/sep1"),            NULL, NULL,    0,       "<Separator>" },
-    { N_("/Edit/_Insert ASCII Value..."), "<control>I", mb_proc, M_ASCII, "<StockItem>", GTK_STOCK_CONVERT },
+    { N_("/Edit/_Insert ASCII Value..."), "<control>I", mb_proc, M_ASCII, NULL },
 
     { N_("/_View"),                NULL, NULL,    0,       "<Branch>" },
     { N_("/View/_Basic Mode"),     "<control>B", mb_proc, M_BASIC, "<RadioItem>" },
@@ -176,6 +179,7 @@ static GtkItemFactoryEntry main_menu[] = {
     { N_("/View/_Scientific Mode"),"<control>S", mb_proc, M_SCI, "/View/Basic Mode" },
     { N_("/View/sep1"),            NULL, NULL,    0,       "<Separator>" },
     { N_("/View/Show _Trailing Zeroes"),"<control>T", mstz_proc, M_ZEROES, "<ToggleItem>" },
+   { N_("/View/Show T_housands Separator"),"<control>K", ts_proc, M_TSEP, "<ToggleItem>" },
     { N_("/View/sep2"),	           NULL, NULL,	  0,       "<Separator>" },
     { N_("/View/_Memory Registers"),"<control>M", mb_proc, M_REGS, "<ToggleItem>" },
 
@@ -285,6 +289,7 @@ about_cb(GtkWidget *widget, gpointer data)
             NULL
         };
         const gchar *documenters[] = {
+            "Sun Microsystems",
             NULL
         };
         const gchar *translator_credits = _("translator_credits");
@@ -463,6 +468,11 @@ button_proc(GtkButton *widget, gpointer user_data)
     } else {
         process_item(n);
     }
+
+    if (v->new_input && v->dtype == FIX) {
+        STRCPY(v->fnum, v->display);
+        set_display(v->fnum);
+    }
 }
 
 
@@ -613,8 +623,7 @@ create_aframe()  /* Create auxiliary frame for ASC key. */
     gtk_widget_show(cancel_button);
     gtk_box_pack_start(GTK_BOX(button_hbox), cancel_button, FALSE, FALSE, 0);
 
-    insert_button = button_new_with_stock_image(_("_Insert"), 
-                                                GTK_STOCK_CONVERT);
+    insert_button = gtk_button_new_with_mnemonic(_("_Insert"));
     g_return_if_fail(insert_button != NULL);
     gtk_widget_ref(insert_button);
     gtk_widget_show(insert_button);
@@ -852,18 +861,21 @@ create_kframe()
     gtk_widget_realize(X->kframe);
     gtk_window_set_title(GTK_WINDOW(X->kframe), _(v->tool_label));
 
-    X->fin_frame = make_but_frame(X->kvbox, X->fin_buttons,
+    X->fin_panel = make_but_panel(X->kvbox, X->fin_buttons,
                                   f_buttons, FROWS, FCOLS, "fin");
-    X->mode_panel = create_mode_panel(X->kvbox);
-    X->sci_frame = make_but_frame(X->kvbox, X->sci_buttons,
-                                  s_buttons, SROWS, SCOLS, "sci");
-    gtk_widget_show(X->fin_frame);
-    gtk_widget_show(X->mode_panel);
-    gtk_widget_show(X->sci_frame);
 
-    X->bas_frame = make_but_frame(X->kvbox, X->bas_buttons,
+    X->mode_panel = create_mode_panel(X->kvbox);
+
+    X->sci_panel = make_but_panel(X->kvbox, X->sci_buttons,
+                                  s_buttons, SROWS, SCOLS, "sci");
+
+    gtk_widget_show(X->fin_panel);
+    gtk_widget_show(X->mode_panel);
+    gtk_widget_show(X->sci_panel);
+
+    X->bas_panel = make_but_panel(X->kvbox, X->bas_buttons,
                                   &b_buttons[0], BROWS, BCOLS, "bas");
-    gtk_widget_show(X->bas_frame);
+    gtk_widget_show(X->bas_panel);
     gtk_window_add_accel_group(GTK_WINDOW(X->kframe), X->kbd_accel);
     grey_buttons(v->base);
     gtk_window_set_icon(GTK_WINDOW(X->kframe), X->icon);
@@ -912,21 +924,17 @@ create_mode_panel(GtkWidget *main_vbox)
     int i;
     AtkObject *access_object;
     GtkWidget *base_hbox, *disp_hbox, *trig_hbox;
-    GtkWidget *row1_hbox, *row2_hbox, *frame, *vbox;
+    GtkWidget *row1_hbox, *row2_hbox, *vbox;
     GSList *base_gr = NULL;
     GSList *disp_gr = NULL;
     GSList *trig_gr = NULL;
 
-    frame = gtk_frame_new(NULL);
-    gtk_widget_show(frame);
-    gtk_container_set_border_width(GTK_CONTAINER(frame), 2);
-    gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, TRUE, 0);
-
     row1_hbox = gtk_hbox_new(FALSE, 0);
     row2_hbox = gtk_hbox_new(FALSE, 0);
 
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_widget_ref(vbox);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
 /* Make Trig. type radio button widgets. */
  
@@ -1011,9 +1019,9 @@ create_mode_panel(GtkWidget *main_vbox)
 
     gtk_box_pack_start(GTK_BOX(vbox), row1_hbox, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), row2_hbox, FALSE, FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(frame), vbox);
+    gtk_box_pack_start(GTK_BOX(main_vbox), vbox, FALSE, TRUE, 0);
 
-    return(frame);
+    return(vbox);
 }
 
 
@@ -1326,8 +1334,7 @@ check_vals(int n, int keyval, int state,
         while (buttons[i].value[j] != 0) {
             if (buttons[i].value[j] == keyval) {
                 if (buttons[i].mods[j] == state) {
-                    g_signal_emit_by_name(GTK_WIDGET(gtk_buttons[i]), 
-                                          "activate");
+                    button_proc(GTK_BUTTON(gtk_buttons[i]), NULL);
                     return(TRUE);
                 }
             }
@@ -1399,6 +1406,7 @@ make_frames()
     n = (struct button *) g_object_get_data(G_OBJECT(BUT_ACC), "button");
     menu = create_menu(n->mtype, n);
     set_accuracy_toggle(v->accuracy);
+    set_show_tsep_toggle(v->show_tsep);
     set_show_zeroes_toggle(v->show_zeroes);
 }
 
@@ -1449,22 +1457,21 @@ set_accessible_name(GtkWidget *widget, struct button button)
 
 
 static GtkWidget *
-make_but_frame(GtkWidget *vbox, GtkWidget **Gtk_buttons,
+make_but_panel(GtkWidget *vbox, GtkWidget **Gtk_buttons,
                struct button buttons[], int rows, int cols, char *tag)
 {
     char *label, name[MAXLINE];
     int i, j, n;
-    GtkWidget *frame, *l;
+    GtkWidget *l;
     GtkWidget *table = gtk_table_new(rows, cols, TRUE);
 
-    frame = gtk_frame_new(NULL);
-    gtk_widget_show(frame);
-    gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
+    gtk_table_set_row_spacings(GTK_TABLE(table), 6);
+    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
+    gtk_container_set_border_width(GTK_CONTAINER(table), 6);
 
     gtk_widget_ref(table);
     gtk_widget_show(table);
-    gtk_container_set_border_width(GTK_CONTAINER(frame), 2);
-    gtk_container_add(GTK_CONTAINER(frame), table);
+    gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
 
     for (i = 0; i < cols; i++) {
         for (j = 0; j < rows; j++) {
@@ -1501,11 +1508,11 @@ make_but_frame(GtkWidget *vbox, GtkWidget **Gtk_buttons,
                      i, i+1, j, j+1,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL | GTK_SHRINK),
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL | GTK_SHRINK),
-                     4, 4);
+                     0, 0);
         }
     }
 
-    return(frame);
+    return(table);
 }
 
 
@@ -1724,7 +1731,6 @@ mstz_proc(gpointer data, int choice, GtkWidget *item)
 {
     GtkWidget *mi;
 
-
     if (!v->doing_mi) {
 	v->show_zeroes = !v->show_zeroes;
         v->doing_mi = 1;
@@ -1757,9 +1763,15 @@ static void
 put_constant(int n, char *con_value, char *con_name)
 {
     char key[MAXLINE];
+    char *cstr = g_strdup(con_value);
+
+/* Constants are written out with no thousands seaparator and with a radix
+ * character of ".".
+ */
 
     SPRINTF(key, "/apps/%s/constant%1dvalue", v->appname, n);
-    gconf_client_set_string(X->client, key, con_value, NULL);
+    gconf_client_set_string(X->client, key, cstr, NULL);
+    g_free(cstr);
 
     SPRINTF(key, "/apps/%s/constant%1dname", v->appname, n);
     gconf_client_set_string(X->client, key, con_name, NULL);
@@ -1809,6 +1821,10 @@ reset_mode_values(enum mode_type mtype)
     v->accuracy = 9;
     set_accuracy_toggle(v->accuracy);
 
+    v->show_tsep = FALSE;
+    set_show_tsep_toggle(v->show_tsep);
+    put_resource(R_TSEP, set_bool(v->show_tsep == TRUE));
+
     v->show_zeroes = FALSE;
     set_show_zeroes_toggle(v->show_zeroes);
     put_resource(R_ZEROES, set_bool(v->show_zeroes == TRUE));
@@ -1840,16 +1856,24 @@ set_accuracy_toggle(int val)
 void
 set_display(char *str)
 {
+    char localized[MAX_LOCALIZED];
     GtkTextBuffer *buffer;
     GtkTextIter start, end;
 
+    if (str == NULL || *str == '.')
+      str = " ";
+    else
+      {
+	localize_number(localized, str);
+	str = localized;
+      }
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(X->display_item));
     gtk_text_buffer_get_bounds(buffer, &start, &end);
     gtk_text_buffer_delete(buffer, &start, &end);
 
     gtk_text_buffer_insert_with_tags_by_name(buffer,
                                              &end,
-                                             str != NULL && strlen(str) != 0 ? str : " ",
+                                             str,
                                              -1,
                                              "x-large",
                                              NULL);
@@ -1906,6 +1930,7 @@ set_error_state(int error)
     SET_MENUBAR_ITEM_STATE(M_FIN,    !v->error); 
     SET_MENUBAR_ITEM_STATE(M_SCI,    !v->error); 
     SET_MENUBAR_ITEM_STATE(M_ZEROES, !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_TSEP,   !v->error); 
     SET_MENUBAR_ITEM_STATE(M_REGS,   !v->error); 
     SET_MENUBAR_ITEM_STATE(M_ABOUT,  !v->error);
 }
@@ -1943,21 +1968,21 @@ set_mode(enum mode_type mode)
 
     switch (mode) {
         case BASIC:
-            gtk_widget_hide(X->fin_frame);
+            gtk_widget_hide(X->fin_panel);
             gtk_widget_hide(X->mode_panel);
-            gtk_widget_hide(X->sci_frame);
+            gtk_widget_hide(X->sci_panel);
             break;
 
         case FINANCIAL:
-            gtk_widget_show(X->fin_frame);
+            gtk_widget_show(X->fin_panel);
             gtk_widget_hide(X->mode_panel);
-            gtk_widget_hide(X->sci_frame);
+            gtk_widget_hide(X->sci_panel);
             break;
 
         case SCIENTIFIC:
-            gtk_widget_hide(X->fin_frame);
+            gtk_widget_hide(X->fin_panel);
             gtk_widget_show_all(X->mode_panel);
-            gtk_widget_show(X->sci_frame);
+            gtk_widget_show(X->sci_panel);
             break;
     }
 
@@ -1968,8 +1993,8 @@ set_mode(enum mode_type mode)
     gtk_widget_size_request(X->display_item, r);
     w = MAX(w, r->width);
     h += r->height;
-    if (GTK_WIDGET_VISIBLE(X->fin_frame)) {
-        gtk_widget_size_request(X->fin_frame, r);
+    if (GTK_WIDGET_VISIBLE(X->fin_panel)) {
+        gtk_widget_size_request(X->fin_panel, r);
         w = MAX(w, r->width);
         h += r->height;
     }
@@ -1978,8 +2003,8 @@ set_mode(enum mode_type mode)
         w = MAX(w, r->width);
         h += r->height;
     }
-    if (GTK_WIDGET_VISIBLE(X->sci_frame)) {
-        gtk_widget_size_request(X->sci_frame, r);
+    if (GTK_WIDGET_VISIBLE(X->sci_panel)) {
+        gtk_widget_size_request(X->sci_panel, r);
         w = MAX(w, r->width);
     	h += r->height;
     }
@@ -2048,6 +2073,16 @@ set_gcalctool_icon(void)
 }
 
 
+static void
+set_show_tsep_toggle(int state)
+{
+    GtkWidget *mi;
+
+    mi = gtk_item_factory_get_widget_by_action(X->mb_fact, M_TSEP);
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi), state);
+}
+
+
 static void
 set_show_zeroes_toggle(int state)
 {
@@ -2098,6 +2133,21 @@ start_tool()
 }
 
 
+static void
+ts_proc(gpointer data, int choice, GtkWidget *item)
+{
+    if (!v->started) {
+	return;
+    }
+
+    v->show_tsep = !v->show_tsep;
+
+    show_display(v->MPdisp_val);
+    put_resource(R_TSEP, set_bool(v->show_tsep == TRUE));
+    make_registers();
+}
+
+
 void
 win_display(enum fcp_type fcptype, int state)
 {
